package quest

import (
	"fmt"
	"strings"
	"validator/conf"

	log "github.com/sirupsen/logrus"
	"gorm.io/driver/postgres"
	"gorm.io/gorm"
)

type Quest struct {
	db *gorm.DB
}

func NewQuest(cfg *conf.QuestConfig) (q *Quest) {
	dsn := fmt.Sprintf("host=%s port=%d user=%s dbname=%s password=%s sslmode=disable", cfg.Host, cfg.Port, cfg.User, cfg.Database, cfg.Password)
	_db, err := gorm.Open(postgres.Open(dsn))
	if err != nil {
		panic(err)
	}
	if cfg.Debug {
		_db = _db.Debug()
	}
	testSQL := "SELECT 1;"
	err = _db.Raw(testSQL).Error
	if err != nil {
		panic(err)
	}
	log.Info("connect quest success")
	return &Quest{db: _db}
}

func (q *Quest) GetProofs(startTimestamp, endTimestamp int64, lastTaskID string, limit int) (proofs []*ProofModel, err error) {
	querySQL := "SELECT " +
		"`TaskType`, `TaskId`, `TaskFinishTimestamp`, `TaskWorkload`, `TaskReqHash`, `TaskRespHash`, `TaskManagerSignature`, `TaskContainerSignature`, `TaskMinerSignature`, `TaskProfitAccount`, `TaskWorkerAccount` " +
		"FROM `proof` " +
		"WHERE `TaskFinishTimestamp` >= ? AND `TaskFinishTimestamp` < ? " +
		"AND `TaskId` > ? " +
		"ORDER BY `TaskId` ASC " +
		"LIMIT ?;"

	err = q.db.Raw(querySQL, startTimestamp, endTimestamp, lastTaskID, limit).Scan(&proofs).Error
	return
}

func (q *Quest) GetPendingWorkload(startTimestamp int64, address string) (workload uint64, err error) {
	proof := new(ProofModel)
	querySQL := "SELECT " +
		"`TaskWorkload`, `TaskReqHash`, `TaskRespHash`, `TaskManagerSignature`, `TaskContainerSignature`, `TaskMinerSignature`, `TaskProfitAccount`, `TaskWorkerAccount` " +
		"FROM `proof` " +
		"WHERE `TaskFinishTimestamp` >= ? " +
		"AND `TaskProfitAccount` = ? ;"

	err = q.db.Raw(querySQL, startTimestamp, address).First(&proof).Error
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return 0, nil
		}
		return 0, err
	}
	return proof.TaskWorkload, nil
}

func (q *Quest) GetGlobalWorkload(startTimestamp int64) (workload uint64, err error) {
	querySQL := "SELECT " +
		"SUM(`TaskWorkload`) " +
		"FROM `proof` " +
		"WHERE `TaskFinishTimestamp` >= ? AND `TaskFinishTimestamp` < ?;"
	err = q.db.Raw(querySQL, startTimestamp, startTimestamp+86400).Row().Scan(&workload)
	if err != nil && strings.Contains(err.Error(), "converting NULL to uint64 is unsupported") {
		return 0, nil
	}
	return workload, err
}
